import config_entrypoint
from loader import Loader, Settings


class ConfigPy2:
    settings = None

    def __init__(self, envvar_prefix='CONFIG', settings_files=['settings.toml'], settings_dir='', dontLoad=False):
        Loader.ENV_PREFIX = envvar_prefix
        Loader.SETTINGS_FILES = settings_files
        Loader.SETTINGS_DIR = settings_dir
        if not dontLoad:
            self.load()

    def __new__(cls):
        return super(ConfigPy2, cls).__new__(cls)

    def load(self):
        # if ConfigPy2.settings:
        #     return
        Loader.load()
        ConfigPy2.settings = Loader.settings



import click, toml, ujson, yaml
from loader import Loader, isfile, Settings


class Help(click.Command):
    def format_help(self, ctx, formatter):
        click.echo('''
Usage: config_py2 [OPTIONS] COMMAND [ARGS]

Options:
  --version         Show config_py2 version currently installed
  --help            Show this message and exit

Commands:
  init              Inits a config_py2 project. Default behavior of using `settings.toml` can be overriden with [ARGS]
  list              Lists all defined config values

Args:   Optionally use EITHER one of these in conjunction with `init` command
  -f, --filename    Pass one or more of this followed by a filename (with extension) to give a list of file(s) to use for config
  -t, --filetype    Pass one or more of this followed by a filetype (extension) to use for settings file.
                    Possible values are: [toml | json | yaml]
                    Please note that this option will not be used if filenames are passed via the -f, --filename flag
  -e, --env_prefix  Pass this option to override the ENV_PREFIX that will be used to override configs with the shell variables
                    Format of environment variables will be `ENV_PREFIX_{}`. By default it is set to \'CONFIG\''''
)


@click.command(cls=Help)
@click.argument('command', type=click.Choice(['init', 'list']))
@click.option('-f', '--filename', default='', multiple=True)
@click.option('-t', '--filetype', default='toml', type=click.Choice(['toml', 'json', 'yaml']))
@click.option('-e', '--env_prefix', default='CONFIG')
@click.version_option()
def entrypoint(command, filename, filetype, env_prefix):
    if command == 'init':
        init(filename, filetype, env_prefix)
    elif command == 'list':
        listConfig()

def init(fileNames, filetype, env_prefix):
    Loader.ENV_PREFIX = env_prefix
    if fileNames:
        Loader.SETTINGS_FILES = list(fileNames)
    else:
        fileNames = ['settings.{0}'.format(filetype)]
        Loader.SETTINGS_FILES = fileNames

    click.echo ("Configuring your environment. Please Wait !")

    for fileName in fileNames:
        if isfile(fileName):
            click.echo("File `{0}` already exists ! Exiting ...".format(fileName))
            exit(1)

        defaultConfig = {
            'default' : {},
            'local': {},
            'testing': {},
            'production': {}
        }
        with open(fileName, 'w') as f:
            dumper = None
            filetype = fileName.split('.')[-1]
            if filetype == 'toml':
                dumper = toml

            elif filetype == 'json':
                dumper = ujson

            elif filetype == 'yaml':
                dumper = yaml
                for key in defaultConfig.keys():
                    defaultConfig[key] = {'APP_ENV': key}

            if filetype != 'toml':
                dumper.dump(defaultConfig, f, indent=4)
            else:
                dumper.dump(defaultConfig, f)
        f.close()
    with open('config_manager.py', 'w') as f:
        f.write(
'''
from config_py2 import ConfigPy2
from os.path import dirname, realpath

settings = ConfigPy2(
    envvar_prefix   = '{0}' ,
    settings_files  = {1}   ,
    settings_dir    = dirname(realpath(__file__))
).settings
'''.format(env_prefix, fileNames)
        )
        f.close()

def listConfig():
    import sys
    from os import getcwd
    sys.path.insert(0, getcwd())
    from config_manager import settings

    click.echo(
        ujson.dumps(
            settings.values(),
            indent=4
        )
    )

if __name__ == '__main__':
    entrypoint()


import click
from os import environ
import toml, ujson, yaml
from os.path import isfile, join

class Settings:
    def __init__(self, env):
        self.CONFIG_ENVIRONMENT = env

    def keys(self):
        return self.__dict__.keys()

    def values(self):
        return self.__dict__

class Loader:
    settings = None
    ENV_PREFIX = 'CONFIG'
    SETTINGS_FILES = ['settings.toml']
    SETTINGS_DIR = ''

    @staticmethod
    def load():
        if not Loader.settings:
            Loader.settings = Settings('default')

        fileNames = Loader.SETTINGS_FILES

        envForConfig = environ.get('CONFIG_ENVIRONMENT')
        if not envForConfig:
            envForConfig = 'default'
        else:
            envForConfig = envForConfig.lower()

        Loader.settings.CONFIG_ENVIRONMENT = envForConfig

        defaultConfig = {}

        for fileName in fileNames:
            filetype = fileName.split('.')[-1]
            dumper = toml if filetype == 'toml' else (ujson if filetype == 'json' else yaml)

            fileName = join(Loader.SETTINGS_DIR, fileName)
            if isfile(fileName):
                with open(fileName, 'r') as f:
                    config = dumper.load(f) if filetype != 'yaml' else dumper.full_load(f)

                    defaultConfig.update(
                        config['default']
                    )

                    for key, value in defaultConfig.iteritems():

                        # Override Config
                        if envForConfig != 'default':
                            key = key.lower()

                            if key in config[envForConfig]:
                                value = config[envForConfig][key]
                            else:
                                key = key.upper()
                                if key in config[envForConfig]:
                                    value = config[envForConfig][key]

                        key = key.upper()
                        val = environ.get('{0}_{1}'.format(Loader.ENV_PREFIX, key))
                        if val:
                            value = val

                        setattr(Loader.settings, key, value)

                    f.close()
            else:
                click.echo("Config File {0} NOT found !!!".format(fileName))
                Loader.settings = None

        del defaultConfig



from os.path import isfile
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()
    fh.close()

if isfile("requirements.txt"):
    requirementsFile = "requirements.txt"
else:
    requirementsFile = "config_py2.egg-info/requires.txt"

with open(requirementsFile, "r") as fh:
    requirements = fh.read()
    fh.close()

VERSION = '1.3.3'

setup(
    name="config_py2",
    version=VERSION,
    author = 'Chalukya',
    author_email = 'chalukya@gmail.com',
    license = 'MIT',
    description = 'A Config Management CLI Tool similar to dynaconf for python 2 and above',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://gitlab.com/chalukyaj/config-manager-python2",
    project_urls = {
        "Bug Tracker": "https://gitlab.com/chalukyaj/config-manager-python2/issues"
    },
    keywords=["config", "cli", "toml", "json", "yaml", "python2", "python 2.x", "dynaconf"],
    install_requires=[requirements],
    packages=['config_py2'],
    python_requires='>=2.7',
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Documentation',
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
    ],
    entry_points={"console_scripts": ["config_py2 = config_py2.config_entrypoint:entrypoint"]},
)
